/*
******************************************************************************

   Fallback stubs for BLT implementation - sink batchop go function.

   Copyright (C) 2001	Brian S. Julin [bri@tull.umassp.edu]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include "stubsblt.h"

static inline void do_switches(struct blt_stubs_row *row, u_int8_t *stack,
			       int x, int y, int w, int *nextx);

int BLT_stubs_sink_go(batchop_t src, batchop_t dest, int numrows) {

	/* We have insider knowlege of the format of the source batchop,
	 * so all we need is the pointer to the row and the index to start at. 
	 */
	int *srcidx;
	struct blt_stubs_row *row;

	srcidx = &(src->out.idx);
	row = src->parms[0].out.ptr.v;

	while (numrows--) {
#warning align this.
		u_int8_t stack[BLT_stubs_stack_pull];
		int y;
		(*srcidx)++;

		y = 0;
		while (y < row->srch) {
			int x, maxw;
			x = 0;
			maxw = row->srcw;
			if (maxw < row->stackw) maxw = row->stackw;
			while (x < row->srcw) {
				int nextx;
				do_switches(row, stack, x, y, maxw, &nextx);
				maxw = row->srcw - nextx;
				if (maxw < row->stackw) maxw = row->stackw;
				x = nextx;
			}
		}

		row++;
	}
	(*srcidx)++;
	bo_set_out_idx(src, *srcidx);
	return 0;
}



static inline void do_switches(struct blt_stubs_row *row, u_int8_t *stack,
			       int x, int y, int w, int *nextx) {
	switch (row->get1) {
	case BLT_STUBS_GET_NIL:
		*nextx = x + w;
		break;
	case BLT_STUBS_GET_VIS:
		ggiGetHLine((ggi_visual_t)(row->get1buf), 
			    row->get1x + x, row->get1y + y, w, 
			    (ggi_pixel *)(stack + row->get1stackoff));
		*nextx = x + w;
		break;
	case BLT_STUBS_GET_PIXBUF_DIRECT:
		memcpy(stack + row->get1stackoff, 
		       (ggi_pixel *)(row->get1buf) + 
		       row->get1x + x + (row->get1y + y) * row->get1stride, 
		       w * sizeof(ggi_pixel));
		*nextx = x + w;
		break;
	}
	switch (row->put1) {
	case BLT_STUBS_PUT_NIL:
		break;
	case BLT_STUBS_PUT_VIS:
		ggiPutHLine((ggi_visual_t)(row->put1buf), 
			    row->put1x + x, row->put1y + y, w, 
			    (ggi_pixel *)(stack + row->put1stackoff));
		break;
	case BLT_STUBS_PUT_PIXBUF_DIRECT:
	        memcpy((ggi_pixel *)(row->put1buf) + 
		       row->put1x + x + (row->put1y + y) * row->put1stride,
		       stack + row->put1stackoff,  
		       w * sizeof(ggi_pixel));
	}
	/* TODO -- shift data to front of stack and provide mechanism for 
	 * not redoing the get when not processing w pixels.
	 */
}
